@@ -6,7 +6,6 @@ import pytest
 
 from click.testing import CliRunner
 
-from ufotest import ufotest
 from ufotest import cli
 
 
@@ -26,7 +25,7 @@ def test_content(response):
     # assert 'GitHub' in BeautifulSoup(response.content).title.string
 
 
-def test_command_line_interface():
+def command_line_interface():
     """Test the CLI."""
     runner = CliRunner()
     result = runner.invoke(cli.main)
