@@ -14,7 +14,7 @@ from ufotest.util import execute_command, get_command_output, execute_script
 
 def pci_write(addr: str, value: str):
     pci_command = 'pci -w {} {}'.format(addr, value)
-    exit_code = execute_command(pci_command, False)
+    exit_code = execute_command(pci_command, verbose=False)
     if exit_code:
         click.secho('Command "{}" failed!'.format(pci_command), fg='red')
 
@@ -94,11 +94,11 @@ def save_frame(path: str, verbose: bool, tmp_path='/tmp'):
     # Decoding the frame into the RAW format
     if verbose:
         click.secho('Decoding the image, with the following settings:')
-        click.secho('-- Camera height:   {}'.format(CONFIG['camera']['sensor_height']))
-        click.secho('-- Camera width:    {}'.format(CONFIG['camera']['sensor_width']))
+        click.secho('-- Camera height:   {}'.format(CONFIG.get_sensor_height()))
+        click.secho('-- Camera width:    {}'.format(CONFIG.get_sensor_width()))
     decode_command = 'ipedec -r {height} --num-columns {width} {path} {verbose}'.format(
-        height=CONFIG['camera']['sensor_height'],
-        width=CONFIG['camera']['sensor_width'],
+        height=CONFIG.get_sensor_height(),
+        width=CONFIG.get_sensor_width(),
         path=data_path,
         verbose='-v' if verbose else ''
     )
