@@ -13,6 +13,7 @@ from jinja2 import Environment, PackageLoader, select_autoescape
 PATH = Path(__file__).parent.absolute()
 
 TEMPLATE_PATH = os.path.join(PATH, 'templates')
+STATIC_PATH = os.path.join(PATH, 'static')
 CONFIG_TEMPLATE_PATH = os.path.join(TEMPLATE_PATH, 'default.toml')
 
 # This will be the string path to the HOME folder of the user which is currently executing the script
@@ -27,9 +28,24 @@ DEFAULT_PATH = os.path.join(HOME_PATH, '.ufotest')
 PATH_ENV = 'UFOTEST_PATH'
 
 
-def get_path():
+def get_builds_path() -> str:
+    """Returns the string path of the 'builds' folder within the ufotest installation folder.
+
+    :return: the string absolute path of the builds folder
     """
-    Returns the path of the folder, where all the global persistent assets of ufotest are being stored.
+    path = get_path()
+    return os.path.join(path, 'builds')
+
+
+def get_path() -> str:
+    """Returns the path of the installation folder of the ufotest app.
+
+    This installation folder contains for example the following things:
+    - The config file for the project. This is a toml file which manages the global config settings for ufotest
+    - The 'archive' folder which saves archives of the test reports of all the executed test runs.
+    - The 'builds' folder which saves archives of all the reports for the issued builds of the remote source repo.
+
+    :return: the string absolute path to the installation folder
     """
     if PATH_ENV in os.environ.keys():
         return os.environ[PATH_ENV]
@@ -101,8 +117,22 @@ class Config(metaclass=Singleton):
     """
 
     def __init__(self):
+        # -- LOAD THE DATA FROM FILE
         self.data = load_config()
 
+        # -- INIT CONTEXT
+        # So here is the scenario: I have noticed that I need to pass some parameters from the click command call
+        # within the terminal all the way down to the email sending sub routine. This is multiple layers and I totally
+        # do not want to modify all of them with an additional parameter.
+        # This is why I have come up with the 'context' idea. I'll just have a global object where I save the necessary
+        # values in and then I can access them in the email subroutine. And the config object is perfect since this
+        # already is a global singleton! I'll just add the additional sub dict "context" which will work exactly as
+        # I have intended
+        self.data['context'] = {
+            'hostname':         self.data['ci']['hostname'],
+            'verbose':          False
+        }
+
     # IMPLEMENTING DICT FUNCTIONALITY
     # -------------------------------
 
@@ -144,6 +174,24 @@ class Config(metaclass=Singleton):
     def get_time_format(self):
         return self.data['general']['time_format']
 
+    def get_ci_repository_url(self):
+        return self.data['ci']['repository_url']
+
+    def get_ci_bitfile_path(self):
+        return self.data['ci']['bitfile_path']
+
+    def get_ci_branch(self):
+        return self.data['ci']['branch']
+
+    def get_ci_suite(self):
+        return self.data['ci']['test_suite']
+
+    def get_email_address(self):
+        return self.data['ci']['gmail_address']
+
+    def get_email_password(self):
+        return self.data['ci']['gmail_password']
+
     # UTILITY METHODS
     # ---------------
 
